package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.KlConceptAllDTO;
import com.diagbot.dto.KlConceptAllSubDTO;
import com.diagbot.dto.KlConceptInfoDTO;
import com.diagbot.dto.KlConceptSimDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptCommon;
import com.diagbot.entity.KlConceptStatic;
import com.diagbot.entity.KlDiagnose;
import com.diagbot.entity.KlDiagnoseBase;
import com.diagbot.entity.KlDrugMapping;
import com.diagbot.entity.KlLexicon;
import com.diagbot.entity.KlLibraryInfo;
import com.diagbot.entity.KlRelation;
import com.diagbot.entity.KlRule;
import com.diagbot.entity.KlRuleBase;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RelationLibTypeEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.KlConceptServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ConceptRelationVO;
import com.diagbot.vo.GetAllForRelationVO;
import com.diagbot.vo.KlConceptAllVO;
import com.diagbot.vo.KlConceptClearVO;
import com.diagbot.vo.KlConceptInfoVO;
import com.diagbot.vo.KlConceptSatarOrdisaVO;
import com.diagbot.vo.KlConceptSaveSubVO;
import com.diagbot.vo.KlConceptSaveVO;
import com.diagbot.vo.KlLibraryInfoVO;
import com.diagbot.vo.SearchConceptVO;
import com.diagbot.vo.SearchVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2021-03-01 13:21
 */
@Component
public class KlConceptFacade extends KlConceptServiceImpl {
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private KlLibraryInfoFacade klLibraryInfoFacade;
    @Autowired
    private KlConceptCommonFacade klConceptCommonFacade;
    @Autowired
    KlLisFacade klLisFacade;
    @Autowired
    KlDiseaseFacade klDiseaseFacade;
    @Autowired
    KlLexiconFacade klLexiconFacade;
    @Autowired
    KlConceptStaticFacade klConceptStaticFacade;
    @Autowired
    KlRelationFacade klRelationFacade;
    @Autowired
    KlRuleFacade klRuleFacade;
    @Autowired
    KlRuleBaseFacade klRuleBaseFacade;
    @Autowired
    KlDiagnoseFacade klDiagnoseFacade;
    @Autowired
    KlDiagnoseBaseFacade klDiagnoseBaseFacade;
    @Autowired
    KlDrugFacade klDrugFacade;
    @Autowired
    KlSymptomFacade klSymptomFacade;
    @Autowired
    KlPacsFacade klPacsFacade;
    @Autowired
    KlOperationFacade klOperationFacade;
    @Autowired
    KlVitalResFacade klVitalResFacade;
    @Autowired
    KlDrugMappingFacade klDrugMappingFacade;
    @Autowired
    KlTcmDiseaseFacade klTcmDiseaseFacade;
    @Autowired
    KlTcmSyndromeFacade klTcmSyndromeFacade;

    /**
     * @param klConceptInfoVO
     * @return
     */
    public IPage<KlConceptInfoDTO> getConceptInfoPage(KlConceptInfoVO klConceptInfoVO) {
        if (StringUtil.isNotBlank(klConceptInfoVO.getSynonymName())) {
            KlLibraryInfoVO klLibraryInfoVO = new KlLibraryInfoVO();
            BeanUtil.copyProperties(klConceptInfoVO, klLibraryInfoVO);
            List<Long> conceptIdList = klLibraryInfoFacade.getLib(klLibraryInfoVO)
                    .stream().map(KlLibraryInfo::getConceptId).distinct().collect(Collectors.toList());
            if (ListUtil.isEmpty(conceptIdList)) {
                conceptIdList.add(-999999999L);
            }
            klConceptInfoVO.setConceptIdList(conceptIdList);
        }
        IPage<KlConceptInfoDTO> conceptPage = this.getConceptPage2(klConceptInfoVO);

        if (ListUtil.isNotEmpty(conceptPage.getRecords())) {
            //获取用户
            List<String> userIds = conceptPage.getRecords().stream()
                    .map(KlConceptInfoDTO::getModifierId).distinct().collect(Collectors.toList());
            RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(userIds);
            RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
            //获取同义词
            List<Long> conceptIds = conceptPage.getRecords().stream()
                    .map(KlConceptInfoDTO::getConceptId).distinct().collect(Collectors.toList());
            QueryWrapper<KlLibraryInfo> qw = new QueryWrapper<>();
            qw.in("concept_id", conceptIds)
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .orderByDesc("concept_id")
                    .orderByDesc("is_concept");
            Map<Long, List<KlLibraryInfo>> libMap = klLibraryInfoFacade.list(qw)
                    .stream().collect(Collectors.groupingBy(KlLibraryInfo::getConceptId));
            //获取类型
            List<Integer> libTypes = conceptPage.getRecords().stream()
                    .map(KlConceptInfoDTO::getLibType).distinct().collect(Collectors.toList());
            QueryWrapper<KlLexicon> qwLib = new QueryWrapper<>();
            qwLib.in("code", libTypes)
                    .eq("is_deleted", IsDeleteEnum.N.getKey());
            Map<Integer, String> lexiconMap = klLexiconFacade.list(qwLib)
                    .stream().collect(Collectors.toMap(KlLexicon::getCode, KlLexicon::getName));
            //组装结果
            conceptPage.getRecords().forEach(i -> {
                i.setTypeName(lexiconMap.get(i.getLibType()));
                i.setSynonymName(libMap.get(i.getConceptId())
                        .stream()
                        .map(KlLibraryInfo::getName)
                        .collect(Collectors.joining("、")));
                i.setModifierName(respDTO.data.get(i.getModifierId()));
            });
        }
        return conceptPage;
    }

    /**
     * @param klConceptAllVO
     * @return
     */
    public KlConceptAllDTO getConceptAlls(KlConceptAllVO klConceptAllVO) {
        Long conceptId = klConceptAllVO.getConceptId();
        List<KlConceptAllDTO> conceptAll = getConceptAll(klConceptAllVO);

        KlConceptAllDTO klConceptAll = new KlConceptAllDTO();
        List<KlConceptAllSubDTO> klConceptSub = new ArrayList<>();
        if (ListUtil.isNotEmpty(conceptAll)) {
            for (KlConceptAllDTO data : conceptAll) {
                if (data.getIsConcept().equals(1)) {
                    BeanUtil.copyProperties(data, klConceptAll);
                } else {
                    KlConceptAllSubDTO klConceptAllSubDTO = new KlConceptAllSubDTO();
                    BeanUtil.copyProperties(data, klConceptAllSubDTO);
                    klConceptSub.add(klConceptAllSubDTO);
                }
            }
        }
        klConceptAll.setKlConceptSub(klConceptSub);

        LexiconEnum lexiconEnum = LexiconEnum.getEnum(klConceptAllVO.getLibType());
        if (lexiconEnum != null) {
            ConceptRelationVO conceptRelationVO = new ConceptRelationVO();
            conceptRelationVO.setConceptId(conceptId);
            switch (lexiconEnum) {
                case Disease: // 疾病扩展 kl_disease, kl_relation，kl_relation_order
                    klDiseaseFacade.getKlDisease(conceptId, klConceptAll);
                    break;
                case Symptom: // 症状扩展 kl_symptom, kl_relation，kl_relation_order
                    klSymptomFacade.getKlSymptom(conceptId, klConceptAll);
                    break;
                case Medicine: // 药品扩展 kl_drug
                    klDrugFacade.getKlDrug(conceptId, klConceptAll);
                    break;
                case LisName:
                case LisSubName: // 化验扩展 kl_lis
                    klLisFacade.getKlLis(conceptId, klConceptAll);
                    break;
                case PacsName:
                case PacsSubName: // 辅检扩展 kl_pacs
                    klPacsFacade.getKlPacs(conceptId, klConceptAll);
                    break;
                case Operation: // 手术扩展 kl_operation
                    klOperationFacade.getKlOperation(conceptId, klConceptAll);
                    break;
                case VitalResult: //体征结果扩展 kl_vital_result, kl_relation，kl_relation_order
                    klVitalResFacade.getKlVitalResult(conceptId, klConceptAll);
                    break;
                case Tcmdisease: // 中医疾病
                    klTcmDiseaseFacade.getKlTcmDisease(conceptId, klConceptAll);
                    break;
                case Tcmsyndrome: // 中医证候
                    klTcmSyndromeFacade.getKlTcmSyndrome(conceptId, klConceptAll);
                    break;
            }
        }
        return klConceptAll;
    }

    /**
     * @param klConceptSaveVO
     * @return
     */
    public Boolean saveConceptInfo(KlConceptSaveVO klConceptSaveVO) {
        Date now = DateUtil.now();

        CommonParam commonParam = initCommonParam();
        boolean res = false;
        // 校验名称是否相同
        checkConcept(klConceptSaveVO);
        //校验
        KlConceptSaveSubVO paesubVO = new KlConceptSaveSubVO();
        paesubVO.setConceptId(klConceptSaveVO.getConceptId());
        paesubVO.setIsConcept(klConceptSaveVO.getIsConcept());
        paesubVO.setLibId(klConceptSaveVO.getLibId());
        paesubVO.setSynonymName(klConceptSaveVO.getSynonymName());
        klLibraryInfoFacade.checkLibraryInfoData(paesubVO);
        //先判断是新加的,还是修改
        if (null != klConceptSaveVO.getConceptId() && null != klConceptSaveVO.getLibId()) {
            commonParam.setConceptId(klConceptSaveVO.getConceptId());
            //更新标准词
            res = updateAll(klConceptSaveVO);
            //更新同义词
            List<KlConceptSaveSubVO> klConceptAllSub = klConceptSaveVO.getKlConceptSub();
            if (klConceptAllSub.size() > 0) {
                for (KlConceptSaveSubVO subVO : klConceptAllSub) {
                    if (subVO.getLibId() == null) {
                        //校验
                        klLibraryInfoFacade.checkLibraryInfoData(subVO);
                        //保存LibraryInfo
                        res = klLibraryInfoFacade.savekLibraryInfoData(klConceptSaveVO.getConceptId(), subVO);
                    } else {
                        //校验
                        klLibraryInfoFacade.checkLibraryInfoData(subVO);
                        UpdateWrapper<KlLibraryInfo> klLibraryInfoQuerySub = new UpdateWrapper<>();
                        klLibraryInfoQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                                .eq("id", subVO.getLibId())
                                .set("gmt_modified", now)
                                .set("modifier", UserUtils.getCurrentPrincipleID())
                                .set("type_id", subVO.getLibType())
                                .set("name", subVO.getSynonymName())
                                .set("spell", subVO.getSpell())
                                .set("remark", subVO.getRemark());
                        res = klLibraryInfoFacade.update(new KlLibraryInfo(), klLibraryInfoQuerySub);
                    }
                }
            }
        } else {
            //校验只能有一条数据
            checkLibType(klConceptSaveVO);
            //保存新数据
            //1.先保存标准词，生成conceptId
            KlConcept klConceptPare = new KlConcept();
            Long libId = klConceptSaveVO.getLibId();
            if (null == libId) {
                libId = Long.valueOf(-(new Random().nextInt(100000)));
            }
            klConceptPare.setLibName(klConceptSaveVO.getLibName());
            klConceptPare.setLibType(klConceptSaveVO.getLibType());
            klConceptPare.setStatus(klConceptSaveVO.getStatus());
            klConceptPare.setLibId(libId);
            klConceptPare.setCreator(UserUtils.getCurrentPrincipleID());
            klConceptPare.setModifier(UserUtils.getCurrentPrincipleID());
            klConceptPare.setGmtCreate(now);
            klConceptPare.setGmtModified(now);
            res = save(klConceptPare);
            commonParam.setConceptId(klConceptPare.getId()); // 赋值conceptId
            if (res) {
                //2.再把标准词d的conceptId保存到kl_library_info表 生成libId，
                Long conceptId = klConceptPare.getId();
                KlLibraryInfo klLibraryInfoSub = new KlLibraryInfo();
                klLibraryInfoSub.setConceptId(commonParam.getConceptId());
                klLibraryInfoSub.setName(klConceptSaveVO.getLibName());
                klLibraryInfoSub.setSpell(klConceptSaveVO.getSpell());
                klLibraryInfoSub.setTypeId(klConceptSaveVO.getLibType());
                klLibraryInfoSub.setIsConcept(klConceptSaveVO.getIsConcept());
                klLibraryInfoSub.setRemark(klConceptSaveVO.getRemark());
                klLibraryInfoSub.setCreator(UserUtils.getCurrentPrincipleID());
                klLibraryInfoSub.setModifier(UserUtils.getCurrentPrincipleID());
                klLibraryInfoSub.setGmtCreate(now);
                klLibraryInfoSub.setGmtModified(now);
                res = klLibraryInfoFacade.save(klLibraryInfoSub);
                if (res) {
                    //3.最后更新回concept表里得lid_id字段
                    UpdateWrapper<KlConcept> klConceptQuerySub = new UpdateWrapper<>();
                    klConceptQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                            .eq("id", conceptId)
                            .set("lib_Id", klLibraryInfoSub.getId());
                    res = update(new KlConcept(), klConceptQuerySub);
                }
            }
            //KlConceptCommon
            if (null != klConceptSaveVO.getIsHasCommon() && klConceptSaveVO.getIsHasCommon().equals(1)) {
                KlConceptCommon conceptCommon = new KlConceptCommon();
                conceptCommon.setSexType(klConceptSaveVO.getSexType());
                conceptCommon.setMaxAge(klConceptSaveVO.getMaxAge());
                conceptCommon.setMinAge(klConceptSaveVO.getMinAge());
                conceptCommon.setConceptId(klConceptPare.getId());
                conceptCommon.setCreator(UserUtils.getCurrentPrincipleID());
                conceptCommon.setModifier(UserUtils.getCurrentPrincipleID());
                conceptCommon.setGmtCreate(now);
                conceptCommon.setGmtModified(now);
                klConceptCommonFacade.save(conceptCommon);
            }
            //增加同义词
            List<KlConceptSaveSubVO> klConceptSub = klConceptSaveVO.getKlConceptSub();
            if (ListUtil.isNotEmpty(klConceptSub)) {
                for (KlConceptSaveSubVO subVO : klConceptSub) {
                    if (subVO.getLibId() == null) {
                        klLibraryInfoFacade.checkLibraryInfoData(subVO);
                        res = klLibraryInfoFacade.savekLibraryInfoData(klConceptPare.getId(), subVO);
                    }
                }
            }
        }

        LexiconEnum lexiconEnum = LexiconEnum.getEnum(klConceptSaveVO.getLibType().intValue());
        if (lexiconEnum != null) {
            switch (lexiconEnum) {
                case Disease: // 疾病扩展 kl_disease, kl_relation，kl_relation_order
                    res = klDiseaseFacade.saveAll(commonParam, klConceptSaveVO.getKlDiseaseVO());
                    klRelationFacade.saveKlDisease(commonParam, klConceptSaveVO.getKlDiseaseVO());
                    break;
                case Symptom: // 症状扩展 kl_symptom, kl_relation，kl_relation_order
                    klSymptomFacade.saveAll(commonParam, klConceptSaveVO.getKlSymptomVO());
                    klRelationFacade.saveKlSymptom(commonParam, klConceptSaveVO.getKlSymptomVO());
                    break;
                case Medicine: //  药品通用名扩展 kl_drug
                    klDrugFacade.saveKlDrug(commonParam, klConceptSaveVO.getKlDrugVO());
                    break;
                case LisName: // 实验室检查套餐
                case LisSubName: // 实验室检查子项目
                    klLisFacade.saveAll(commonParam, klConceptSaveVO.getKlLisVO());
                    break;
                case PacsSubName: // 辅助检查子项目
                case PacsName: // 辅助检查项目
                    klPacsFacade.saveAll(commonParam, klConceptSaveVO.getKlPacsVO());
                    break;
                case Operation: // 手术和操作
                    klOperationFacade.saveAll(commonParam, klConceptSaveVO.getKlOperationVO());
                    break;
                case VitalResult: // 体征结果
                    klRelationFacade.saveKlVitalRes(commonParam, klConceptSaveVO.getKlVitalResultVO());
                    break;
                case Tcmdisease: // 中医疾病
                    klTcmDiseaseFacade.saveAll(commonParam, klConceptSaveVO.getKlTcmDiseaseVO());
                    break;
                case Tcmsyndrome: // 中医证候
                    klTcmSyndromeFacade.saveAll(commonParam, klConceptSaveVO.getKlTcmSyndromeVO());
                    break;
            }
        }
        return res;
    }

    public Boolean updateAll(KlConceptSaveVO klConceptSaveVO) {
        Date now = DateUtil.now();
        boolean res = false;
        UpdateWrapper<KlConcept> klConceptQuerySub = new UpdateWrapper<>();
        klConceptQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSaveVO.getConceptId())
                .set("gmt_modified", now)
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("lib_name", klConceptSaveVO.getLibName());
        res = update(new KlConcept(), klConceptQuerySub);

        UpdateWrapper<KlLibraryInfo> klLibraryInfoQuerySub = new UpdateWrapper<>();
        klLibraryInfoQuerySub.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSaveVO.getLibId())
                .set("gmt_modified", now)
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("name", klConceptSaveVO.getLibName())
                .set("spell", klConceptSaveVO.getSpell())
                .set("remark", klConceptSaveVO.getRemark());

        res = klLibraryInfoFacade.update(new KlLibraryInfo(), klLibraryInfoQuerySub);
        //更新kl_concept_common表
        if (null != klConceptSaveVO.getIsHasCommon() && klConceptSaveVO.getIsHasCommon().equals(1)) {
            //当是疾病的时候才保存性别和年龄大小的数据
            res = klConceptCommonFacade.saveOrUpdateAll(klConceptSaveVO);
        }
        return res;
    }

    /**
     * 获取所有概念(术语关系维护时筛选使用)
     *
     * @param getAllForRelationVO
     * @return
     */
    public List<GetAllForRelationDTO> getAllForRelation(GetAllForRelationVO getAllForRelationVO) {
        List<GetAllForRelationDTO> retList = Lists.newArrayList();

        if (StringUtil.isNotBlank(getAllForRelationVO.getName())
                || null != getAllForRelationVO.getTypeId()) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(getAllForRelationVO.getName()), "lib_name", getAllForRelationVO.getName().trim())
                    .eq(getAllForRelationVO.getTypeId() != null, "lib_type", getAllForRelationVO.getTypeId())
                    .eq("status", StatusEnum.Enable.getKey())
                    .notIn(ListUtil.isNotEmpty(getAllForRelationVO.getExcludedConceptIds()), "id", getAllForRelationVO.getExcludedConceptIds())
            );
            //            retList = BeanUtil.listCopyTo(conceptList, GetAllForRelationDTO.class);
            if (ListUtil.isNotEmpty(conceptList)) {
                retList = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptNameType(String.valueOf(x.getLibType()));
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }

        }
        return retList;
    }

    /**
     * 根据类型和名称查找术语，名称和类型不能为空
     *
     * @param searchConceptVO
     * @return
     */
    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchConceptVO searchConceptVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = Lists.newArrayList();
        String name = searchConceptVO.getName();
        Integer libType = searchConceptVO.getLibType();
        List<Long> excludedConceptIds = searchConceptVO.getExcludedConceptIds();
        if (StringUtils.isNotBlank(name) && libType != null) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(name), "lib_name", name.trim())
                    .eq("lib_type", libType)
                    .eq("status", StatusEnum.Enable.getKey())
                    .notIn(ListUtil.isNotEmpty(excludedConceptIds), "id", excludedConceptIds));
            if (ListUtil.isNotEmpty(conceptList)) {
                getAllForRelationDTOS = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptNameType(x.getLibName());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }
        }
        return getAllForRelationDTOS;
    }

    /**
     * 根据类型和名称查找术语，名称可为空
     *
     * @param SearchVO
     * @return
     */
    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchVO SearchVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = Lists.newArrayList();
        List<Long> excludedConceptIds = SearchVO.getExcludedConceptIds();
        List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", SearchVO.getLibType())
                .eq("status", StatusEnum.Enable.getKey())
                .like(StringUtil.isNotBlank(SearchVO.getName()), "lib_name", SearchVO.getName())
                .notIn(ListUtil.isNotEmpty(excludedConceptIds), "id", excludedConceptIds));
        if (ListUtil.isNotEmpty(conceptList)) {
            getAllForRelationDTOS = conceptList.stream().map(x -> {
                GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                getAllForRelationDTO.setConceptNameType(x.getLibName());
                getAllForRelationDTO.setConceptName(x.getLibName());
                getAllForRelationDTO.setConceptId(x.getId());
                getAllForRelationDTO.setLibType(x.getLibType());
                return getAllForRelationDTO;
            }).collect(Collectors.toList());
        }
        return getAllForRelationDTOS;
    }

    /**
     * 筛选符合类型的概念id
     *
     * @param libTypeId
     * @param sourceConceptIds
     * @return
     */
    public List<Long> getCompatibleTypeConceptIds(Long libTypeId, List<Long> sourceConceptIds) {
        if (libTypeId == null || ListUtil.isEmpty(sourceConceptIds)) {
            return null;
        }
        QueryWrapper<KlConcept> conceptQe = new QueryWrapper<>();
        conceptQe.in("id", sourceConceptIds);
        conceptQe.eq("lib_type", libTypeId);
        return list(conceptQe).stream().map(i -> i.getId()).collect(Collectors.toList());
    }

    public Boolean clearConceptInfo(KlConceptClearVO klConceptClearVO) {
        Date now = DateUtil.now();
        int sum = klLibraryInfoFacade.count(new QueryWrapper<KlLibraryInfo>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptClearVO.getLibId()).eq("is_concept", 0));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存在或删除的不是同义词！");
        }
        UpdateWrapper<KlLibraryInfo> klLibraryUpdate = new UpdateWrapper<>();
        klLibraryUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptClearVO.getLibId()).set("is_deleted", IsDeleteEnum.Y.getKey()).set("gmt_modified", now);
        return klLibraryInfoFacade.update(klLibraryUpdate);
    }

    public Boolean startOrDisableRule(KlConceptSatarOrdisaVO klConceptSatarOrdisaVO, int status) {
        Date now = DateUtil.now();
        int sum = this.count(new QueryWrapper<KlConcept>().eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSatarOrdisaVO.getConceptId()));
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该数据已不存！");
        }
        if (status == StatusEnum.Disable.getKey()) {
            //kl_concept_static 静态信息
            int conceptStaticSum = klConceptStaticFacade.count(new QueryWrapper<KlConceptStatic>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSatarOrdisaVO.getConceptId()));
            if (conceptStaticSum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与静态信息存在关系！");
            }

            //kl_relation
            List<Integer> notRelationList = Lists.newArrayList(RelationLibTypeEnum.relationDept.getKey(),
                    RelationLibTypeEnum.relationPart.getKey(), RelationLibTypeEnum.relationSystem.getKey(),
                    RelationLibTypeEnum.relationAccSymptom.getKey(), RelationLibTypeEnum.relationNature.getKey());
            int relationsum = klRelationFacade.count(new QueryWrapper<KlRelation>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .and(r -> r.eq("end_id", klConceptSatarOrdisaVO.getConceptId())
                            .or(r1 -> r1.eq("start_id", klConceptSatarOrdisaVO.getConceptId()).notIn("relation_id", notRelationList))
                    )
            );
            if (relationsum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与树形结构或疾病相关或扩展信息存在关系！");
            }
            //kl_rule
            int rulesum = klRuleFacade.count(new QueryWrapper<KlRule>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSatarOrdisaVO.getConceptId()));
            if (rulesum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与规则维护存在关系！");
            }
            //kl_rule_base
            int ruleBasesum = klRuleBaseFacade.count(new QueryWrapper<KlRuleBase>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSatarOrdisaVO.getConceptId()));
            if (ruleBasesum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与规则维护明细存在关系！");
            }
            //kl_diagnose
            int diagnosesum = klDiagnoseFacade.count(new QueryWrapper<KlDiagnose>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSatarOrdisaVO.getConceptId()));
            if (diagnosesum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与诊断依据维护存在关系！");
            }
            //kl_diagnose_base
            int diagnoseBasesum = klDiagnoseBaseFacade.count(new QueryWrapper<KlDiagnoseBase>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("concept_id", klConceptSatarOrdisaVO.getConceptId()));
            if (diagnoseBasesum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与诊断依据维护明细存在关系！");
            }
            //kl_drug_mapping
            int drugMappingSum = klDrugMappingFacade.count(new QueryWrapper<KlDrugMapping>().eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("drug_concept", klConceptSatarOrdisaVO.getConceptId()));
            if (drugMappingSum > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该医学标准术语与注册药品存在关系！");
            }
        }
        UpdateWrapper<KlConcept> klLibraryUpdate = new UpdateWrapper<>();
        klLibraryUpdate.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", klConceptSatarOrdisaVO.getConceptId())
                .set("status", status).set("gmt_modified", now);
        // .set("modifier", UserUtils.getCurrentPrincipleID()
        // );
        return this.update(klLibraryUpdate);

    }

    private void checkLibType(KlConceptSaveVO klConceptSaveVO) {
        QueryWrapper<KlLexicon> klLexiconQuery = new QueryWrapper<>();
        klLexiconQuery.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("only_one", 1);
        List<KlLexicon> klLexiconList = klLexiconFacade.list(klLexiconQuery);
        List<Integer> libTypeList = klLexiconList.stream().map(KlLexicon::getCode).collect(Collectors.toList());
        if (libTypeList.contains(klConceptSaveVO.getLibType())) {
            int count = this.count(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("lib_type", klConceptSaveVO.getLibType()));
            if (count > 0) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "该术语类型只能有一个");
            }
        }
    }

    public void checkConcept(KlConceptSaveVO klConceptSaveVO) {
        // 校验名称是否相同
        int count = this.count(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", klConceptSaveVO.getLibType())
                .eq("lib_name", klConceptSaveVO.getLibName())
                .ne("id", klConceptSaveVO.getConceptId() == null ? -1 : klConceptSaveVO.getConceptId()));
        if (count > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "医学标准术语名称已存在");
        }
        //校验标准词是否存在同义词
        int countLib = klLibraryInfoFacade.count(new QueryWrapper<KlLibraryInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("name", klConceptSaveVO.getLibName())
                .eq("type_id", klConceptSaveVO.getLibType())
                .eq("is_concept", 0));
        if (countLib > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "医学标准术语名称已存在同义词");
        }
    }

    /**
     * 根据conceptId和relationId获取关联数据（通用方法）
     *
     * @param conceptRelationVO
     * @return
     */
    public Map<Integer, List<KlConceptSimDTO>> getRelationConceptFac(ConceptRelationVO conceptRelationVO) {
        Map<Integer, List<KlConceptSimDTO>> map = Maps.newLinkedHashMap();
        List<KlConceptSimDTO> relationConcept = this.getRelationConcept(conceptRelationVO);
        if (ListUtil.isNotEmpty(relationConcept)) {
            map = EntityUtil.makeEntityListMap(relationConcept, "libType");
        }
        return map;
    }

    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        String person = UserUtils.getCurrentPrincipleID();
        param.setNow(DateUtil.now());
        param.setPerson(person);
        return param;
    }
}
