package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptDetail;
import com.diagbot.entity.KlConceptStatic;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.service.impl.KlConceptStaticServiceImpl;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 15:41
 */
@Component
public class KlConceptStaticFacade extends KlConceptStaticServiceImpl {

    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private KlConceptDetailFacade klConceptDetailFacade;
    @Autowired
    private KlConceptFacade klConceptFacade;

    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();

        RespDTO<List<StaticKnowledgeIndexDTO>> respDTO = cdssCoreClient.staticKnowledgeIndex(staticKnowledgeIndexVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }


    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO staticKnowledgeDTO = new StaticKnowledgeDTO();

        RespDTO<StaticKnowledgeDTO> respDTO = cdssCoreClient.getStaticKnowledge(staticKnowledgeVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            staticKnowledgeDTO = respDTO.data;
        }
        return staticKnowledgeDTO;
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Page<KlConceptStaticDTO> page = null;
        RespDTO<Page<KlConceptStaticDTO>> respDTO = cdssCoreClient.getPage(klConceptStaticPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        }
        List<KlConceptStaticDTO> records = page.getRecords();
        List<String> userIds = records.stream().map(KlConceptStaticDTO::getModifier)
                .distinct()
                .collect(Collectors.toList());
        RespDTO<Map<String, String>> userRespDTO = userServiceClient.getUserInfoByIds(userIds);
        if (RespDTOUtil.respIsNG(userRespDTO)) {
            return page;
        }
        Map<String, String> userMap = userRespDTO.data;
        records.forEach(record -> {
            if (userMap.containsKey(record.getModifier())) {
                record.setModifier(userMap.get(record.getModifier()));
            } else {
                record.setModifier("");
            }
        });
        page.setRecords(records);
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klConceptStaticVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateRecord(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        }
        return success;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        }
        return success;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();
        RespDTO<KlConceptStaticDTO> respDTO = cdssCoreClient.getRecordById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klConceptStaticDTO = respDTO.data;
        }
        return klConceptStaticDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Boolean exist = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.isExist(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            exist = respDTO.data;
        }
        return exist;
    }

    /**
     * 获取全静态知识
     *
     * @return
     */
    public List<KlConcept> getAllConcept() {
        List<KlConcept> retList = Lists.newLinkedList();
        QueryWrapper<KlConceptStatic> staticQueryWrapper = new QueryWrapper<>();
        staticQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("status", StatusEnum.Enable.getKey());
        List<KlConceptStatic> staticList = this.list(staticQueryWrapper);

        if (ListUtil.isNotEmpty(staticList)) {
            List<Long> conceptIds = staticList.stream().map(i -> i.getConceptId()).distinct().collect(Collectors.toList());
            QueryWrapper<KlConceptDetail> detailQueryWrapper = new QueryWrapper<>();
            detailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds);
            List<KlConceptDetail> detailsList = klConceptDetailFacade.list(detailQueryWrapper);
            if (ListUtil.isNotEmpty(detailsList)) {
                conceptIds = detailsList.stream().map(i -> i.getConceptId()).distinct().collect(Collectors.toList());
                retList = klConceptFacade.list(new QueryWrapper<KlConcept>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("status", StatusEnum.Enable.getKey())
                        .in("id", conceptIds));
            }
        }
        return retList;
    }
}