package com.diagbot.facade;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.DiagnosesFindDTO;
import com.diagbot.dto.KlDiagnoseInfoDTO;
import com.diagbot.dto.KlDiagnoseTypeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.KlDiagnose;
import com.diagbot.service.impl.KlDiagnoseServiceImpl;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.DiagnosesFindVO;
import com.diagbot.vo.KlDiagnoseByIdVO;
import com.diagbot.vo.KlDiagnoseClearVO;
import com.diagbot.vo.KlDiagnoseInfoVO;
import com.diagbot.vo.KlDiagnoseSatarDisVO;
import com.diagbot.vo.KlDiagnoseSaveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2021-03-23 16:54
 */
@Component
public class KlDiagnoseFacade extends KlDiagnoseServiceImpl {
    @Autowired
    CdssCoreClient cdssCoreClient;
    @Autowired
    UserServiceClient userServiceClient;

    public Page<KlDiagnoseInfoDTO> getKlDiagnoseInfoPage(KlDiagnoseInfoVO klDiagnoseInfoVO) {
        RespDTO<Page<KlDiagnoseInfoDTO>> diagnosePages = cdssCoreClient.getDiagnosePages(klDiagnoseInfoVO);
        RespDTOUtil.respNGDeal(diagnosePages, "获取分页获取诊断依据列表数据失败");
        Page<KlDiagnoseInfoDTO> records = diagnosePages.data;
        List<String> userIds = records.getRecords().stream()
                .map(i -> i.getModifier()).distinct().collect(Collectors.toList());
        RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(userIds);
        RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
        records.getRecords().forEach(i -> {
            i.setModifierName(respDTO.data.get(i.getModifier()));
        });
        return records;

    }

    public List<KlDiagnoseTypeDTO> getByIdDiagnoseAll(KlDiagnoseByIdVO klDiagnoseByIdVO) {
        RespDTO<List<KlDiagnoseTypeDTO>> byIdDiagnoseAll = cdssCoreClient.getByIdDiagnoseAll(klDiagnoseByIdVO);
        RespDTOUtil.respNGDeal(byIdDiagnoseAll, "根据Id获取诊断依据详情数据失败");
        return byIdDiagnoseAll.data;
    }


    public Boolean saveDiagnoseAll(KlDiagnoseSaveVO klDiagnoseSaveVO) {
        klDiagnoseSaveVO.setModifier(UserUtils.getCurrentPrincipleID());
        RespDTO<Boolean> booleanRespDTO = cdssCoreClient.saveDiagnoseAll(klDiagnoseSaveVO);
        RespDTOUtil.respNGDeal(booleanRespDTO, "保存诊断依据数据失败");
        return booleanRespDTO.data;
    }

    public Boolean clearDiagnoseAll(KlDiagnoseClearVO klDiagnoseClearVO) {
        RespDTO<Boolean> booleanRespDTO = cdssCoreClient.clearDiagnoseAll(klDiagnoseClearVO);
        RespDTOUtil.respNGDeal(booleanRespDTO, "删除诊断依据失败");
        return booleanRespDTO.data;
    }

    public Boolean disableDiagnose(KlDiagnoseSatarDisVO klDiagnoseSatarDisVO) {
        klDiagnoseSatarDisVO.setModifier(UserUtils.getCurrentPrincipleID());
        RespDTO<Boolean> booleanRespDTO = cdssCoreClient.disableDiagnoses(klDiagnoseSatarDisVO);
        RespDTOUtil.respNGDeal(booleanRespDTO, "停用诊断依据失败");
        return booleanRespDTO.data;

    }

    public Boolean startKlDiagnoses(KlDiagnoseSatarDisVO klDiagnoseSatarDisVO) {
        klDiagnoseSatarDisVO.setModifier(UserUtils.getCurrentPrincipleID());
        RespDTO<Boolean> booleanRespDTO = cdssCoreClient.startKlDiagnoses(klDiagnoseSatarDisVO);
        RespDTOUtil.respNGDeal(booleanRespDTO, "启用诊断依据失败");
        return booleanRespDTO.data;
    }

    public List<DiagnosesFindDTO> findDiaNameAlls(DiagnosesFindVO diagnosesFindVO) {
        RespDTO<List<DiagnosesFindDTO>> booleanRespDTO = cdssCoreClient.findDiaNameAll(diagnosesFindVO);
        RespDTOUtil.respNGDeal(booleanRespDTO, "搜索未添加的诊断依据术语名失败");
        return booleanRespDTO.data;
    }
}
