package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.KlConceptSimDTO;
import com.diagbot.dto.KlDrugExportDTO;
import com.diagbot.dto.KlDrugRegisterDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlDrugMapping;
import com.diagbot.entity.KlDrugRegister;
import com.diagbot.enums.InsertOrUpdateEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.KlDrugMappingService;
import com.diagbot.service.KlDrugRegisterService;
import com.diagbot.service.impl.KlDrugRegisterServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.KlDrugMappingGetVO;
import com.diagbot.vo.KlDrugRegisterDelVO;
import com.diagbot.vo.KlDrugRegisterGetVO;
import com.diagbot.vo.KlDrugRegisterPageVO;
import com.diagbot.vo.KlDrugRegisterSaveVO;
import com.diagbot.vo.KlDrugRegisterTestVO;
import com.diagbot.vo.KlDrugSearchVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.map.LinkedMap;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zhoutg
 * @Description:
 * @date 2021-04-13 10:41
 */
@Component
public class KlDrugRegisterFacade extends KlDrugRegisterServiceImpl {

    @Autowired
    @Qualifier("klDrugMappingServiceImpl")
    KlDrugMappingService klDrugMappingService;
    @Autowired
    KlConceptFacade klConceptFacade;
    @Autowired
    @Qualifier("klDrugRegisterServiceImpl")
    KlDrugRegisterService klDrugRegisterService;
    @Autowired
    UserFacade userFacade;
    @Autowired
    UserClientFacade userClientFacade;

    /**
     * 保存或更新
     *
     * @param klDrugRegisterSaveVO
     */
    public void saveOrUpdate(KlDrugRegisterSaveVO klDrugRegisterSaveVO) {
        CommonParam param = initCommonParam();
        KlDrugRegister klDrugRegister = new KlDrugRegister();
        if (klDrugRegisterSaveVO.getId() != null) {
            klDrugRegister = this.getOne(new QueryWrapper<KlDrugRegister>()
                    .eq("id", klDrugRegisterSaveVO.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()), false);
            if (klDrugRegister == null) { //校验
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "数据不存在");
            }
            param.setInsertOrUpdate(InsertOrUpdateEnum.Update.getKey());
        }
        BeanUtil.copyProperties(klDrugRegisterSaveVO, klDrugRegister);
        // 重复校验
        // int count = this.count(new QueryWrapper<KlDrugRegister>()
        //         .eq("is_deleted", IsDeleteEnum.N.getKey())
        //         .eq("drug_code", klDrugRegisterSaveVO.getDrugCode())
        //         .ne("id", klDrugRegisterSaveVO.getId() == null ? -1 : klDrugRegisterSaveVO.getId()));
        // if (count > 0) {
        //     throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "药品代码已存在");
        // }
        if (InsertOrUpdateEnum.Insert.getKey() == param.getInsertOrUpdate()) {
            klDrugRegister.setCreator(param.getPerson()); //创建人
            klDrugRegister.setGmtCreate(param.getNow());//创建时间
        }
        klDrugRegister.setGmtModified(param.getNow());//修改时间
        klDrugRegister.setModifier(param.getPerson());//修改人
        this.saveOrUpdate(klDrugRegister);

        // 保存药品通用名和注册名映射关系
        klDrugMappingService.remove(new QueryWrapper<KlDrugMapping>()
                .eq("register_id", klDrugRegister.getId()));

        // 保存药品通用名和注册药品的关联
        if (klDrugRegister.getId() != null) {
            KlDrugMapping klDrugMapping = new KlDrugMapping();
            klDrugMapping.setRegisterId(klDrugRegister.getId());
            klDrugMapping.setDrugConcept(klDrugRegisterSaveVO.getDrugConceptId());
            klDrugMapping.setCreator(param.getPerson());
            klDrugMapping.setModifier(param.getPerson());
            klDrugMapping.setGmtModified(param.getNow());
            klDrugMapping.setGmtCreate(param.getNow());
            klDrugMappingService.save(klDrugMapping);
        }
    }

    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        String person = UserUtils.getCurrentPrincipleID();
        param.setNow(DateUtil.now());
        param.setPerson(person);
        param.setInsertOrUpdate(InsertOrUpdateEnum.Insert.getKey());
        return param;
    }

    /**
     * 分页
     *
     * @param klDrugRegisterPageVO
     * @return
     */
    public IPage<KlDrugRegisterDTO> getPageFac(KlDrugRegisterPageVO klDrugRegisterPageVO) {
        IPage<KlDrugRegisterDTO> res = this.getPage(klDrugRegisterPageVO);

        // 设置操作人姓名
        userClientFacade.setUserName(res.getRecords());
        return res;
    }

    /**
     * 根据id删除标签
     *
     * @param klDrugRegisterDelVO
     * @return
     */
    public void deleteByIdFac(KlDrugRegisterDelVO klDrugRegisterDelVO) {
        List<KlDrugMapping> klDrugMappingList = klDrugMappingService.list(new QueryWrapper<KlDrugMapping>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("register_id", klDrugRegisterDelVO.getId()));
        if (ListUtil.isNotEmpty(klDrugMappingList)) {
            // 校验是否有关联
            // List<Long> idList = klDrugMappingList.stream().map(r -> r.getDrugConcept()).collect(Collectors.toList());
            // List<KlConcept> klConceptList = klConceptFacade.list(new QueryWrapper<KlConcept>()
            //         .eq("is_deleted", IsDeleteEnum.N.getKey()).in("id", idList));
            // if (ListUtil.isNotEmpty(klConceptList)) {
            //     String errMsg = klConceptList.stream().map(r -> r.getLibName()).distinct().collect(Collectors.joining("，"));
            //     if (StringUtil.isNotBlank(errMsg)) {
            //         throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "已关联药品通用名：" + errMsg);
            //     }
            // }
            klDrugMappingService.remove(new QueryWrapper<KlDrugMapping>().in("register_id", klDrugRegisterDelVO.getId()));
        }
        this.remove(new QueryWrapper<KlDrugRegister>().eq("id", klDrugRegisterDelVO.getId()));
    }

    /**
     * 根据参数返回数据
     *
     * @param klDrugRegisterGetVO 获取标签内容参数
     * @return 标签内容
     */
    public KlDrugRegisterDTO getByParam(KlDrugRegisterGetVO klDrugRegisterGetVO) {
        KlDrugRegisterDTO klDrugRegisterDTO = new KlDrugRegisterDTO();
        KlDrugRegister klDrugRegister = this.getOne(new QueryWrapper<KlDrugRegister>()
                .eq("id", klDrugRegisterGetVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()), false);
        if (klDrugRegister != null) {
            BeanUtil.copyProperties(klDrugRegister, klDrugRegisterDTO);
        }

        // 获取注册对应的药品通用名
        KlDrugMappingGetVO klDrugMappingGetVO = new KlDrugMappingGetVO();
        klDrugMappingGetVO.setRegisterId(klDrugRegister.getId());
        List<KlConceptSimDTO> klConceptSimDTOList = this.getMappingDrug(klDrugMappingGetVO);
        if (ListUtil.isNotEmpty(klConceptSimDTOList)) {
            KlConceptSimDTO conceptSimDTO = klConceptSimDTOList.get(0);
            klDrugRegisterDTO.setDrugName(conceptSimDTO.getLibName());
            klDrugRegisterDTO.setDrugConceptId(conceptSimDTO.getId());
        }
        return klDrugRegisterDTO;
    }

    /**
     * 药品注册名检索药品通用名
     *
     * @param klDrugSearchVO
     * @return
     */
    public List<KlConceptSimDTO> searchDrugFac(KlDrugSearchVO klDrugSearchVO) {
        return this.searchDrug(klDrugSearchVO);
    }

    /**
     * 注册药品导入
     *
     * @param file
     * @return
     */
    public void importRegister(MultipartFile file, HttpServletResponse response) {
        Map res = new LinkedMap();
        Long t1 = System.currentTimeMillis();
        List<KlDrugRegisterTestVO> list = XLSXCovertCSVReader.readData(file, "注册药品",
                30, KlDrugRegisterTestVO.class);
        Long t2 = System.currentTimeMillis();
        res.put("读取sheet耗时:", (t2 - t1) / 1000.0 + "秒");
        try {
            List<KlConcept> klConceptList = klConceptFacade.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey()).eq("lib_type", 101));
            Map<String, Long> druMap = klConceptList.stream().collect(Collectors.toMap(k -> k.getLibName(), v -> v.getId(), (r1, r2) -> (r2)));
            List<String> errMsg = Lists.newArrayList();
            List<KlDrugRegister> drugRegisterList = Lists.newLinkedList();
            // int i = 2;
            for (KlDrugRegisterTestVO  klDrugRegisterTestVO : list) {
                KlDrugRegister klDrugRegister = new KlDrugRegister();
                BeanUtil.copyProperties(klDrugRegisterTestVO, klDrugRegister);
                String drugName = klDrugRegisterTestVO.getDrugName();
                klDrugRegister.setRemark(drugName);
                drugRegisterList.add(klDrugRegister);
                if (druMap.get(drugName) == null) {
                    errMsg.add(drugName);
                }
                // if (StringUtil.isBlank(drugName)) {
                //     errMsg.add(String.format("第%s行药品通用名为空", i));
                // } else if (druMap.get(drugName) == null) {
                //     errMsg.add(String.format("第%s行【%s】不是标准词", i, drugName));
                // }
                // i++;
            }
            if (ListUtil.isNotEmpty(errMsg)) {
                errMsg = errMsg.stream().distinct().collect(Collectors.toList());
                List<KlDrugExportDTO> klDrugExportDTOList = Lists.newLinkedList();
                for (String s : errMsg) {
                    KlDrugExportDTO klDrugExportDTO = new KlDrugExportDTO();
                    klDrugExportDTO.setName(s);
                    klDrugExportDTOList.add(klDrugExportDTO);
                }
                ExcelUtils.exportExcel(klDrugExportDTOList, null, "药品通用名", KlDrugExportDTO.class,
                        "未校验通过的药品通用名", response, 12.8f);
                res.put("错误", StringUtils.join(errMsg, "；"));
                return;
            }
            this.remove(new QueryWrapper<KlDrugRegister>()); // 删除主表
            Long t3 = System.currentTimeMillis();
            klDrugRegisterService.saveBatch(drugRegisterList);
            Long t4 = System.currentTimeMillis();
            res.put("保存DrugRegister表耗时:", (t4 - t3) / 1000.0 + "秒");

            List<KlDrugMapping> klDrugMappingList = ListUtil.newArrayList();
            for (KlDrugRegister klDrugRegister : drugRegisterList) {
                KlDrugMapping klDrugMapping = new KlDrugMapping();
                klDrugMapping.setRegisterId(klDrugRegister.getId());
                klDrugMapping.setDrugConcept(druMap.get(klDrugRegister.getRemark()));
                klDrugMappingList.add(klDrugMapping);
            }
            klDrugMappingService.remove(new QueryWrapper<KlDrugMapping>());
            Long t5 = System.currentTimeMillis();
            klDrugMappingService.saveBatch(klDrugMappingList);
            Long t6 = System.currentTimeMillis();
            res.put("保存mapping表耗时:", (t6 - t5) / 1000.0 + "秒");
        } catch (Exception e) {
            e.printStackTrace();
        }
        Long end = System.currentTimeMillis();
        res.put("总耗时:", (end - t1) / 1000.0 + "秒");
    }

    public static void main(String[] args) {

    }
}
