package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RelationDTO;
import com.diagbot.dto.TreeAllDTO;
import com.diagbot.dto.TreeDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.KlRelation;
import com.diagbot.entity.KlRelationOrder;
import com.diagbot.entity.KlVitalResult;
import com.diagbot.enums.RelationLibTypeEnum;
import com.diagbot.service.KlRelationOrderService;
import com.diagbot.service.KlRelationService;
import com.diagbot.service.KlVitalResultService;
import com.diagbot.service.impl.KlRelationServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.KlDiseaseVO;
import com.diagbot.vo.KlSymptomVO;
import com.diagbot.vo.KlVitalResultVO;
import com.diagbot.vo.RelationVO;
import com.diagbot.vo.TreeVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/3/3 15:44
 */
@Component
public class KlRelationFacade extends KlRelationServiceImpl {

    @Autowired
    @Qualifier("klRelationOrderServiceImpl")
    KlRelationOrderService klRelationOrderService;

    @Autowired
    @Qualifier("klRelationServiceImpl")
    KlRelationService klRelationService;

    @Autowired
    KlConceptFacade klConceptFacade;

    @Qualifier("klVitalResultServiceImpl")
    @Autowired
    KlVitalResultService klVitalResultService;
;
    public TreeAllDTO getTree(TreeVO treeVO) {
        List<Long[]> pList = new ArrayList<>();
        pList.add(new Long[] { 402L, 600L, 301L, 600L, 101L });//药品化学物质类别
        pList.add(new Long[] { 403L, 600L, 302L, 600L, 101L });//药品治疗学类别
        pList.add(new Long[] { 404L, 600L, 303L, 600L, 101L });//药品药理学类别
        pList.add(new Long[] { 405L, 600L, 304L, 600L, 101L });//药品解剖学类别
        pList.add(new Long[] { 406L, 600L, 305L, 600L, 103L });//症状类别
        pList.add(new Long[] { 407L, 600L, 306L, 600L, 106L });//手术和操作类别
        pList.add(new Long[] { 408L, 600L, 107L, 600L, 108L });//实验室检查类别
        pList.add(new Long[] { 409L, 600L, 109L, 600L, 110L });//辅助检查类别

        Integer index = treeVO.getType() - 1;
        List<Long> types = new ArrayList<>();
        types.add(pList.get(index)[0]);
        types.add(pList.get(index)[1]);
        types.add(pList.get(index)[2]);
        types.add(pList.get(index)[3]);
        types.add(pList.get(index)[4]);

        RelationVO relationVO = new RelationVO();
        relationVO.setSType(pList.get(index)[0]);
        relationVO.setRId(pList.get(index)[1]);
        relationVO.setEType(pList.get(index)[2]);
        List<RelationDTO> root = this.getRelations(relationVO);
        TreeAllDTO treeAllDTO = new TreeAllDTO();
        treeAllDTO.setTypes(types);

        if (ListUtil.isNotEmpty(root)) {
            RelationVO relationVO2 = new RelationVO();
            relationVO2.setSType(pList.get(index)[2]);
            relationVO2.setRId(pList.get(index)[3]);
            relationVO2.setEType(pList.get(index)[4]);
            List<RelationDTO> sub = this.getRelations(relationVO2);
            Map<Long, List<RelationDTO>> map = EntityUtil.makeEntityListMap(sub, "sId");

            TreeDTO treeDTO = new TreeDTO();
            treeDTO.setId(root.get(0).getSId());
            treeDTO.setName(root.get(0).getSName());
            for (RelationDTO relationDTO : root) {
                TreeDTO treeDTO2 = new TreeDTO();
                treeDTO2.setId(relationDTO.getEId());
                treeDTO2.setName(relationDTO.getEName());
                List<RelationDTO> e = map.get(relationDTO.getEId());
                if (ListUtil.isNotEmpty(e)) {
                    for (RelationDTO relationDTO1 : e) {
                        TreeDTO treeDTO3 = new TreeDTO();
                        treeDTO3.setId(relationDTO1.getEId());
                        treeDTO3.setName(relationDTO1.getEName());
                        treeDTO2.getNextTree().add(treeDTO3);
                    }
                }
                treeDTO.getNextTree().add(treeDTO2);
            }
            treeAllDTO.setTreeDTO(treeDTO);
        }
        return treeAllDTO;
    }

    /**
     * 疾病扩展
     *
     * @param commonParam
     * @param klDiseaseVO
     */
    public void saveKlDisease(CommonParam commonParam, KlDiseaseVO klDiseaseVO) {
        // 疾病关联科室，部位，关联系统统一处理。先查找全部数据，删除关联排序表，再删除关联表
        List<Integer> relationIdList = Lists.newArrayList(
                RelationLibTypeEnum.relationDept.getKey(),
                RelationLibTypeEnum.relationPart.getKey(),
                RelationLibTypeEnum.relationSystem.getKey()
        );
        klRelationRemove(commonParam, relationIdList);

        if (klDiseaseVO != null) {
            // 【科室关联】
            klRelationSave(commonParam, klDiseaseVO.getDeptList(), RelationLibTypeEnum.relationDept.getKey());
            // 【部位关联】
            klRelationSave(commonParam, klDiseaseVO.getPartList(), RelationLibTypeEnum.relationPart.getKey());
            // 【疾病系统分类关联】
            klRelationSave(commonParam, klDiseaseVO.getSystemTypeList(), RelationLibTypeEnum.relationSystem.getKey());
        }
    }

    /**
     * 症状扩展
     *
     * @param commonParam
     * @param klSymptomVO
     */
    public void saveKlSymptom(CommonParam commonParam, KlSymptomVO klSymptomVO) {
        // 症状关联科室，部位，伴随症状，性质统一处理。先查找全部数据，删除关联排序表，再删除关联表
        List<Integer> relationIdList = Lists.newArrayList(
                RelationLibTypeEnum.relationDept.getKey(),
                RelationLibTypeEnum.relationPart.getKey(),
                RelationLibTypeEnum.relationAccSymptom.getKey(),
                RelationLibTypeEnum.relationNature.getKey()
        );
        klRelationRemove(commonParam, relationIdList);

        if (klSymptomVO != null) {
            // 【科室关联】
            klRelationSave(commonParam, klSymptomVO.getDeptList(), RelationLibTypeEnum.relationDept.getKey());
            // 【部位关联】
            klRelationSave(commonParam, klSymptomVO.getPartList(), RelationLibTypeEnum.relationPart.getKey());
            // 【常见伴随症状关联】
            klRelationSave(commonParam, klSymptomVO.getConSymptomList(), RelationLibTypeEnum.relationAccSymptom.getKey());
            // 【性质关联】
            klRelationSave(commonParam, klSymptomVO.getNatureList(), RelationLibTypeEnum.relationNature.getKey());
        }
    }

    /**
     * 体征结果扩展
     * @param commonParam
     * @param klVitalResultVO
     */
    public void saveKlVitalRes(CommonParam commonParam, KlVitalResultVO klVitalResultVO) {
        if(klVitalResultVO == null){
            return;
        }
        // 先删除体征结果扩展表
        klVitalResultService.remove(new QueryWrapper<KlVitalResult>().eq("concept_id", commonParam.getConceptId()));
        // 重新插入扩展表
        KlVitalResult klVitalResult = new KlVitalResult();
        BeanUtil.copyProperties(klVitalResultVO, klVitalResult);
        klVitalResult.setConceptId(commonParam.getConceptId());
        klVitalResult.setGmtModified(commonParam.getNow());
        klVitalResult.setGmtCreate(commonParam.getNow());
        klVitalResult.setCreator(commonParam.getPerson());
        klVitalResult.setModifier(commonParam.getPerson());
        klVitalResultService.save(klVitalResult);

        // 体征结果关联科室，部位统一处理。先查找全部数据，删除关联排序表，再删除关联表
        List<Integer> relationIdList = Lists.newArrayList(
                RelationLibTypeEnum.relationDept.getKey(),
                RelationLibTypeEnum.relationPart.getKey()
        );
        klRelationRemove(commonParam, relationIdList);

        if (klVitalResultVO != null) {
            // 【科室关联】
            klRelationSave(commonParam, klVitalResultVO.getDeptList(), RelationLibTypeEnum.relationDept.getKey());
            // 【部位关联】
            klRelationSave(commonParam, klVitalResultVO.getPartList(), RelationLibTypeEnum.relationPart.getKey());
        }
    }

    /**
     * 删除关联表和关联排序表
     *
     * @param commonParam 通用参数
     * @param relationList 关联类型列表
     */
    public void klRelationRemove(CommonParam commonParam, List<Integer> relationList) {
        List<KlRelation> klRelationList = this.list(new QueryWrapper<KlRelation>()
                .eq("start_id", commonParam.getConceptId())
                .in("relation_id", relationList)
        );
        if (ListUtil.isNotEmpty(klRelationList)) {
            List<Long> idList = klRelationList.stream().map(r -> r.getId()).collect(Collectors.toList());
            // 删除关联排序表
            klRelationOrderService.remove(new QueryWrapper<KlRelationOrder>().in("t_relation_id", idList));
            // 删除关联表
            this.remove(new QueryWrapper<KlRelation>().in("id", idList));
        }
    }

    /**
     * 保存关联表和关联排序表
     *
     * @param commonParam
     * @param idList
     * @param relationId
     */
    public void klRelationSave(CommonParam commonParam, List<Long> idList, Integer relationId) {
        if (ListUtil.isEmpty(idList)) {
            return;
        }
        List<KlRelation> relationSaveList = Lists.newArrayList();
        for (Long endId : idList) {
            KlRelation klRelation = new KlRelation();
            klRelation.setStartId(commonParam.getConceptId());
            klRelation.setRelationId(relationId);
            klRelation.setEndId(endId);
            klRelation.setGmtModified(commonParam.getNow());
            klRelation.setGmtCreate(commonParam.getNow());
            klRelation.setCreator(commonParam.getPerson());
            klRelation.setModifier(commonParam.getPerson());
            relationSaveList.add(klRelation);
        }
        klRelationService.saveBatch(relationSaveList);

        // 【科室关联排序】
        List<KlRelationOrder> OrderSaveList = Lists.newArrayList();
        int i = 1;
        for (KlRelation klRelation : relationSaveList) {
            KlRelationOrder klRelationOrder = new KlRelationOrder();
            klRelationOrder.settRelationId(klRelation.getId());
            klRelationOrder.setOrderNo(i++);
            klRelation.setGmtModified(commonParam.getNow());
            klRelation.setGmtCreate(commonParam.getNow());
            klRelation.setCreator(commonParam.getPerson());
            klRelation.setModifier(commonParam.getPerson());
            OrderSaveList.add(klRelationOrder);
        }
        klRelationOrderService.saveBatch(OrderSaveList);
    }


}
