package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.KlConceptAllDTO;
import com.diagbot.dto.KlConceptSimDTO;
import com.diagbot.dto.KlSymptomDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.KlSymptom;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RelationLibTypeEnum;
import com.diagbot.service.impl.KlSymptomServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.ConceptRelationVO;
import com.diagbot.vo.KlSymptomVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author zhoutg
 * @Description:
 * @date 2021-04-13 10:41
 */
@Component
public class KlSymptomFacade extends KlSymptomServiceImpl {

    @Autowired
    KlConceptFacade klConceptFacade;

    /**
     * 保存症状扩展表
     *
     * @param commonParam
     * @param klSymptomVO
     * @return
     */
    public boolean saveAll(CommonParam commonParam, KlSymptomVO klSymptomVO) {
        // 先删除疾病扩展表
        this.remove(new QueryWrapper<KlSymptom>().eq("concept_id", commonParam.getConceptId()));
        // 重新插入扩展表
        KlSymptom klSymptom = new KlSymptom();
        BeanUtil.copyProperties(klSymptomVO, klSymptom);
        klSymptom.setConceptId(commonParam.getConceptId());
        klSymptom.setGmtModified(commonParam.getNow());
        klSymptom.setGmtCreate(commonParam.getNow());
        klSymptom.setCreator(commonParam.getPerson());
        klSymptom.setModifier(commonParam.getPerson());
        return this.save(klSymptom);
    }

    /**
     * 疾病扩展获取
     *
     * @param conceptId
     * @param klConceptAllDTO
     */
    public void getKlSymptom(Long conceptId, KlConceptAllDTO klConceptAllDTO) {
        KlSymptom klSymptom = this.getOne(new QueryWrapper<KlSymptom>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", conceptId), false);
        if (klSymptom == null) {
            return;
        }
        KlSymptomDTO klSymptomDTO = new KlSymptomDTO();
        BeanUtil.copyProperties(klSymptom, klSymptomDTO);

        List<Integer> relationIdList = Lists.newArrayList(
                RelationLibTypeEnum.relationDept.getKey(),
                RelationLibTypeEnum.relationPart.getKey(),
                RelationLibTypeEnum.relationAccSymptom.getKey(),
                RelationLibTypeEnum.relationNature.getKey()
        );
        ConceptRelationVO conceptRelationVO = new ConceptRelationVO();
        conceptRelationVO.setConceptId(conceptId);
        conceptRelationVO.setRelationList(relationIdList);
        Map<Integer, List<KlConceptSimDTO>> map = klConceptFacade.getRelationConceptFac(conceptRelationVO);
        if (map.get(LexiconEnum.Part.getKey()) != null) { // 关联部位
            klSymptomDTO.setPartList(map.get(LexiconEnum.Part.getKey()));
        }
        if (map.get(LexiconEnum.Dept.getKey()) != null) { // 关联科室
            klSymptomDTO.setDeptList(map.get(LexiconEnum.Dept.getKey()));
        }
        if (map.get(LexiconEnum.Symptom.getKey()) != null) { // 关联伴随症状
            klSymptomDTO.setConSymptomList(map.get(LexiconEnum.Symptom.getKey()));
        }
        if (map.get(LexiconEnum.Nature.getKey()) != null) { // 关联性质
            klSymptomDTO.setNatureList(map.get(LexiconEnum.Nature.getKey()));
        }
        klConceptAllDTO.setKlSymptomDTO(klSymptomDTO);
    }
}
