package com.diagbot.facade;

import com.diagbot.aggregate.RuleTestAggregate;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RuleDTO;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.entity.wrapper.MappingConfigWrapper;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.HospitalIdVO;
import com.diagbot.vo.RuleTestVO;
import com.diagbot.vo.RuleVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/6 11:53
 */
@Component
public class RuleTestFacade {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private RuleTestAggregate ruleTestAggregate;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    @Autowired
    private MappingConfigFacade mappingConfigFacade;

    @Autowired
    private KlRuleFacade klRuleFacade;
    @Autowired
    private UnUsedMappingFacade unUsedMappingFacade;

    /**
     * 全规则测试
     *
     * @param hospitalIdVO
     */
    public Boolean allRuleTest(HospitalIdVO hospitalIdVO) {
        Boolean retFlag = false;
        //开单合理性规则-通用
        RuleTestVO billRuleVO = new RuleTestVO();
        billRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        billRuleVO.setCaseId(1L);
        //开单合理性规则-输血
        RuleTestVO billTransfusionRuleVO = new RuleTestVO();
        billTransfusionRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        billTransfusionRuleVO.setCaseId(2L);
        //危急值规则-检验
        RuleTestVO criticalLisRuleVO = new RuleTestVO();
        criticalLisRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        criticalLisRuleVO.setCaseId(3L);
        //危急值规则-检查
        RuleTestVO criticalPacsRuleVO = new RuleTestVO();
        criticalPacsRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        criticalPacsRuleVO.setCaseId(4L);
        //高危药品
        RuleTestVO highriskDrugRuleVO = new RuleTestVO();
        highriskDrugRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        highriskDrugRuleVO.setCaseId(5L);
        //高危手术
        RuleTestVO highriskOperationRuleVO = new RuleTestVO();
        highriskOperationRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        highriskOperationRuleVO.setCaseId(6L);
        //化验其他值提醒
        RuleTestVO otherLisRuleVO = new RuleTestVO();
        otherLisRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherLisRuleVO.setCaseId(7L);
        //辅检其他值提醒
        RuleTestVO otherPacsRuleVO = new RuleTestVO();
        otherPacsRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherPacsRuleVO.setCaseId(8L);
        //输血其他值提醒
        RuleTestVO otherTransfusionRuleVO = new RuleTestVO();
        otherTransfusionRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        otherTransfusionRuleVO.setCaseId(9L);
        //正常项重复开单
        RuleTestVO reBillRuleVO = new RuleTestVO();
        reBillRuleVO.setHospitalId(hospitalIdVO.getHospitalId());
        reBillRuleVO.setCaseId(15L);

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("billRuleVO", billRuleVO);
            invokeParams.put("billTransfusionRuleVO", billTransfusionRuleVO);
            invokeParams.put("criticalLisRuleVO", criticalLisRuleVO);
            invokeParams.put("criticalPacsRuleVO", criticalPacsRuleVO);
            invokeParams.put("highriskDrugRuleVO", highriskDrugRuleVO);
            invokeParams.put("highriskOperationRuleVO", highriskOperationRuleVO);
            invokeParams.put("otherLisRuleVO", otherLisRuleVO);
            invokeParams.put("otherPacsRuleVO", otherPacsRuleVO);
            invokeParams.put("otherTransfusionRuleVO", otherTransfusionRuleVO);
            invokeParams.put("reBillRuleVO", reBillRuleVO);

            retFlag
                    = dataBeanAggregateQueryFacade.get("ruleTest", invokeParams, Boolean.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }

        return retFlag;
    }

    /**
     * 开单合理性测试
     *
     * @param ruleTestVO
     */
    public Boolean billRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.billRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 开单合理性测试-输血
     *
     * @param ruleTestVO
     */
    public Boolean billTransfusionRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.billTransfusionRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检验
     *
     * @param ruleTestVO
     */
    public Boolean criticalLisRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.criticalLisRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检查
     *
     * @param ruleTestVO
     */
    public Boolean criticalPacsRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.criticalPacsRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 高危药品测试
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean highriskDrugRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.highriskDrugRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 高危手术测试
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean highriskOperationRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.highriskOperationRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-化验
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherLisRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherLisRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-辅检
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherPacsRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherPacsRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-输血
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean otherTransfusionRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.otherTransfusionRuleTest(ruleTestVO);
        return true;
    }

    /**
     * 正常项重复开单
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean reBillRuleTest(RuleTestVO ruleTestVO) {
        ruleTestAggregate.reBillRuleTest(ruleTestVO);
        return true;
    }


    /**
     * 数据导出
     *
     * @param response
     * @param hospitalIdVO
     */
    public void exportExcel(HttpServletResponse response, HospitalIdVO hospitalIdVO) {
        Long hospitalId = hospitalIdVO.getHospitalId();

        List<RuleDTO> list = klRuleFacade.getRules(new RuleVO());

        List<MappingConfigWrapper> lisConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.LisPack.getKey());
        List<MappingConfigWrapper> pacsConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.Pacs.getKey());
        List<MappingConfigWrapper> drugConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.Drug.getKey());
        List<MappingConfigWrapper> diseaseConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.Disease.getKey());
        List<MappingConfigWrapper> operationConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.Operation.getKey());
        List<MappingConfigWrapper> transfusionConfigList = mappingConfigFacade.getListByHospitalIdAndType(hospitalId, ConceptTypeEnum.Transfusion.getKey());


        //诊断映射关系
        Map<String, Map<String, Map<String, List<String>>>> diseaseConfigMap
                = mappingConfigFacade.groupByUniqueNameWithName(null, ConceptTypeEnum.Disease.getKey(), hospitalId);
        //Map<String, Map<String, List<Long>>> diseaseConfigMap = diseaseConfigFacade.getUniqueConfigMap(hospitalId, null, null);

        Map<String, List<Long>> usedMap = unUsedMappingFacade.usedConceptIds(list);

        List<Long> lisIds = usedMap.get("lis");
        List<Long> pacsIds = usedMap.get("pacs");
        List<Long> diseaseIds = usedMap.get("disease");
        List<Long> drugIds = usedMap.get("drug");
        List<Long> operationIds = usedMap.get("operation");
        List<Long> transfusionIds = usedMap.get("transfusion");

        List<String> lisNames = Lists.newLinkedList();
        List<String> pacsNames = Lists.newLinkedList();
        List<String> diseaseNames = Lists.newLinkedList();
        List<String> drugNames = Lists.newLinkedList();
        List<String> operationNames = Lists.newLinkedList();
        List<String> transfusionNames = Lists.newLinkedList();

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(-1);
        conceptVO.setIds(lisIds);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            lisNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(pacsIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            pacsNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(diseaseIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            diseaseNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(drugIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            drugNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(pacsIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            pacsNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(operationIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            operationNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }
        conceptVO.setIds(transfusionIds);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (ListUtil.isNotEmpty(respDTO.data)) {
            transfusionNames = respDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
        }


        diseaseNames.addAll(diseaseConfigMap.keySet().stream().filter(i -> i.contains("流产")).collect(Collectors.toList()));

        List<LisConfig> unUsedLisConfigs
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(lisConfigList, lisNames), LisConfig.class);
        List<PacsConfig> unUsedPacsConfigs
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(pacsConfigList, pacsNames), PacsConfig.class);
        List<DiseaseConfig> unUsedDiseaseConfigs
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(diseaseConfigList, diseaseNames), DiseaseConfig.class);
        List<DrugConfig> unUsedDrugConfigs
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(drugConfigList, drugNames), DrugConfig.class);
        List<OperationConfig> unUsedOperationConfig
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(operationConfigList, operationNames), OperationConfig.class);
        List<TransfusionConfig> unUsedTransfusionConfigs
                = BeanUtil.listCopyTo(unUsedMappingFacade.unUsedMapping(transfusionConfigList, transfusionNames), TransfusionConfig.class);

        String fileName = "规则未使用的映射关系.xls";
        Map<String, Map<Class<?>, List<?>>> map = new HashMap<>();
        Map<Class<?>, List<?>> lisMap = new HashMap<Class<?>, List<?>>();
        lisMap.put(LisConfig.class, unUsedLisConfigs);
        map.put("检验", lisMap);
        Map<Class<?>, List<?>> pacsMap = new HashMap<Class<?>, List<?>>();
        pacsMap.put(PacsConfig.class, unUsedPacsConfigs);
        map.put("检查", pacsMap);
        Map<Class<?>, List<?>> diseaseMap = new HashMap<Class<?>, List<?>>();
        diseaseMap.put(DiseaseConfig.class, unUsedDiseaseConfigs);
        map.put("诊断", diseaseMap);
        Map<Class<?>, List<?>> drugMap = new HashMap<Class<?>, List<?>>();
        drugMap.put(DrugConfig.class, unUsedDrugConfigs);
        map.put("药品", drugMap);
        Map<Class<?>, List<?>> operationMap = new HashMap<Class<?>, List<?>>();
        operationMap.put(OperationConfig.class, unUsedOperationConfig);
        map.put("手术和操作", operationMap);
        Map<Class<?>, List<?>> transfusionMap = new HashMap<Class<?>, List<?>>();
        transfusionMap.put(TransfusionConfig.class, unUsedTransfusionConfigs);
        map.put("输血", transfusionMap);

        ExcelUtils.exportExcel(map, null, fileName, response);
    }
}