package com.diagbot.web;


import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.KlConceptAllDTO;
import com.diagbot.dto.KlConceptInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.KlConceptFacade;
import com.diagbot.vo.KlConceptAllVO;
import com.diagbot.vo.KlConceptClearVO;
import com.diagbot.vo.KlConceptInfoVO;
import com.diagbot.vo.KlConceptSatarOrdisaVO;
import com.diagbot.vo.KlConceptSaveVO;
import com.diagbot.vo.SearchVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 概念表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2021-03-01
 */
@RestController
@RequestMapping("/graph/klConcept")
@Api(value = "新-基础术语维护相关API", tags = { "基础术语维护相关API" })
@SuppressWarnings("unchecked")
public class KlConceptController {
    @Autowired
    KlConceptFacade klConceptFacade;

    @ApiOperation(value = "分页获取基础术语维护列表[by:wangfeng]")
    @PostMapping("/getConceptInfoPage")
    @SysLogger("getConceptInfoPage")
    public RespDTO<IPage<KlConceptInfoDTO>> getConceptInfoPages(@RequestBody KlConceptInfoVO klConceptInfoVO) {
        return RespDTO.onSuc(klConceptFacade.getConceptInfoPage(klConceptInfoVO));
    }

    @ApiOperation(value = "根据id获取基础术语详情[by:wangfeng]")
    @PostMapping("/getConceptAll")
    @SysLogger("getConceptAll")
    public RespDTO<KlConceptAllDTO> getConceptInfoAll(@RequestBody @Valid KlConceptAllVO klConceptAllVO) {
        return RespDTO.onSuc(klConceptFacade.getConceptAlls(klConceptAllVO));
    }

    @ApiOperation(value = "新增和修改基础术语详情[by:wangfeng]",
            notes = "conceptId：术语Id\n" +
                    "libId：libId\n" +
                    "spell：拼音\n" +
                    "synonymName：同义词名称\n" +
                    "isConcept：是否标准词,1:是,0:否\n" +
                    "sexType：性别\n" +
                    "minAge：最小年龄\n" +
                    "maxAge：最大年龄\n" +
                    "remark：备注\n" +
                    "klConceptSub：同义词列表\n" +

                    "<br>" +
                    "【klDiseaseVO：疾病扩展信息】\n" +
                    "icd10Code：ICD10编号\n" +
                    "course：病程\n" +
                    "inducement：诱因\n" +
                    "foodProhibition：饮食禁忌\n" +
                    "hazard：危险因素\n" +
                    "healing：治愈性\n" +
                    "pernicious：危害性\n" +
                    "clinicType：临床分类\n" +
                    "vulArea：好发地区\n" +
                    "vulCrowd：好发人群\n" +
                    "incidence：发病率\n" +
                    "isInfect：是否传染（0：否，1：是）\n" +
                    "complication：并发症\n" +
                    "pathogeny：病因\n" +
                    "disType：疾病分型\n" +
                    "chWestern：中西医疾病（0：通用，1：西，2：中）\n" +
                    "isCommonDis：是否常见病（0：否，1：是）\n" +
                    "isHeredity：是否遗传（0：否，1：是）\n" +
                    "nameSimple：简称\n" +
                    "enNameSimple：英文简称\n" +
                    "enName：英文名称\n" +
                    "tumorCellType：肿瘤细胞类型\n" +
                    "morphology：形态学分类代码\n" +
                    "deptList：科室列表\n" +
                    "partList：部位列表\n" +
                    "systemTypeList：疾病系统分类列表\n" +

                    "<br>" +
                    "【klDrugVO：药品通用名扩展】\n" +
                    "drug：药品类别（0：西药，1：中成药）\n" +

                    "<br>" +
                    "【klSymptomVO：症状扩展信息】\n" +
                    "isPositive：阳性症状（0：否，1：是）\n" +
                    "enName：英文名称\n" +
                    "definition：定义\n" +
                    "chWestern：中西医症状（0：通用，1：西，2：中）\n" +
                    "mitigatingFac：缓解因素\n" +
                    "aggravateFac：加重因素\n" +
                    "cause：症状起因\n" +
                    "commonDis：常见疾病\n" +
                    "preMeasures：预防措施\n" +
                    "foodProhibition：饮食禁忌\n" +
                    "commonExplain：通俗解释\n" +
                    "deptList：科室列表\n" +
                    "partList：发病部位列表\n" +
                    "conSymptomList：常见伴随症状列表\n" +
                    "natureList：性质列表\n" +

                    "<br>" +
                    "【klLisVO：化验扩展信息】\n" +
                    "minValue：最小值\n" +
                    "maxValue：最大值\n" +
                    "type：类型(0:范围内;1:范围外)\n" +
                    "unit：单位概念id\n" +
                    "checkSpecimen：检查标本\n" +
                    "checkMethod：检查方法\n" +
                    "influenceFac：影响因素\n" +
                    "clinicalSig：临床意义\n" +
                    "qualitative：定性（阴、阳性）\n" +
                    "category：所属类别\n" +
                    "enName：英文名称\n" +

                    "<br>" +
                    "【klPacsVO：辅检扩展信息】\n" +
                    "checkMethod：检查方法\n" +
                    "definition：项目定义\n" +
                    "checkObjective：检查目的\n" +
                    "checkSkillful：检查技术\n" +
                    "condLimit：禁忌症和局限性\n" +
                    "interventional：适应症\n" +
                    "emergencySign：危急标识（0：不危急，1：危急）\n" +
                    "checkPrepare：检查前准备\n" +

                    "<br>" +
                    "【klOperationVO：手术扩展信息】\n" +
                    "operationLevel：手术级别（0,1,2,3,4）\n" +
                    "operationCode：手术操作码\n" +

                    "<br>" +
                    "【klVitalResultVO：体征结果扩展信息】\n" +
                    "suitablePopNo：不适宜人群\n" +
                    "checkMethod：操作方法\n" +
                    "deptList：科室列表\n" +
                    "partList：发病部位\n" +

                    "<br>" +
                    "【klTcmDiseaseVO：中医疾病扩展信息】\n" +
                    "code：编码\n" +

                    "<br>" +
                    "【klTcmSyndromeVO：中医症候扩展信息】\n" +
                    "code：编码\n"
                     )
    @PostMapping("/saveConceptInfo")
    @SysLogger("saveConceptInfo")
    @DSTransactional
    public RespDTO<Boolean> saveConceptInfos(@RequestBody @Valid KlConceptSaveVO klConceptSaveVO) {
        return RespDTO.onSuc(klConceptFacade.saveConceptInfo(klConceptSaveVO));
    }

    @ApiOperation(value = "删除同义词[by:wangfeng]")
    @PostMapping("/clearConceptInfo")
    @SysLogger("clearConceptInfo")
    @DSTransactional
    public RespDTO<Boolean> clearConceptInfos(@RequestBody @Valid KlConceptClearVO klConceptClearVO) {
        return RespDTO.onSuc(klConceptFacade.clearConceptInfo(klConceptClearVO));
    }
    @ApiOperation(value = "停用术语[by:wangfeng]")
    @PostMapping("/disableConcept")
    @SysLogger("disableConcept")
    public RespDTO<Boolean> disableConcept(@RequestBody @Valid KlConceptSatarOrdisaVO klConceptSatarOrdisaVO) {
        return RespDTO.onSuc(klConceptFacade.startOrDisableRule(klConceptSatarOrdisaVO, StatusEnum.Disable.getKey()));
    }

    @ApiOperation(value = "启用术语[by:wangfeng]")
    @PostMapping("/startConcept")
    @SysLogger("startConcept")
    public RespDTO<Boolean> startRuleInfos(@RequestBody @Valid KlConceptSatarOrdisaVO klConceptSatarOrdisaVO) {
        return RespDTO.onSuc(klConceptFacade.startOrDisableRule(klConceptSatarOrdisaVO,StatusEnum.Enable.getKey()));
    }

    @ApiOperation(value = "根据类型和名称查找术语，名称可为空[by:zhoutg]",
            notes = "name: 术语的名称<br>" +
                    "libType: 术语的词性<br>" +
                    "excludedConceptIds: 需要排除的概念id集合")
    @PostMapping("/search")
    @SysLogger("search")
    public RespDTO<GetAllForRelationDTO> search(@RequestBody SearchVO SearchVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = klConceptFacade.searchConceptByNameAndLibType(SearchVO);
        return RespDTO.onSuc(getAllForRelationDTOS);
    }
}
