package com.diagbot.web;


import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.KlConceptSimDTO;
import com.diagbot.dto.KlDrugRegisterDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.KlDrugRegisterFacade;
import com.diagbot.vo.KlDrugRegisterDelVO;
import com.diagbot.vo.KlDrugRegisterGetVO;
import com.diagbot.vo.KlDrugRegisterPageVO;
import com.diagbot.vo.KlDrugRegisterSaveVO;
import com.diagbot.vo.KlDrugSearchVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 标签基础表 前端控制器
 * </p>
 *
 * @author zhoutg
 * @since 2018-11-23
 */
@RestController
@RequestMapping("/klDrugRegister")
@SuppressWarnings("unchecked")
@Api(value = "注册药品相关API", tags = { "注册药品相关API" })
public class KlDrugRegisterController {

    @Autowired
    KlDrugRegisterFacade klDrugRegisterFacade;

    @ApiOperation(value = "新增或更新[by:zhoutg]",
            notes = "id：主键\n" +
                    "drugCode：药品代码\n" +
                    "name：注册名称\n" +
                    "enName：英文名称\n" +
                    "tradeName：商品名称\n" +
                    "form：药品剂型\n" +
                    "specification：注册规格\n" +
                    "minPackQuantity：最小包装数量\n" +
                    "minPackUnit：最小包装单位\n" +
                    "company：药品企业\n" +
                    "approval：批准文号\n" +
                    "standardCode：药品本位码\n" +
                    "insuranceType：医保类型\n" +
                    "insuranceRemrk：医保备注\n" +
                    "drugType：药品类别\n" +
                    "remark：备注\n" +
                    "drugConceptId：药品通用名Id")
    @PostMapping("/saveOrUpdate")
    @SysLogger("saveOrUpdate")
    @DSTransactional
    public RespDTO<Boolean> saveOrUpdate(@RequestBody KlDrugRegisterSaveVO klDrugRegisterSaveVO) {
        klDrugRegisterFacade.saveOrUpdate(klDrugRegisterSaveVO);
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "分页[by:zhoutg]",
            notes = "name：注册名称| 商品名称\n" +
                    "form：药品剂型\n" +
                    "company：药品企业\n" +
                    "insuranceType：医保类型\n" +
                    "drugName：药品通用名")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<IPage<KlDrugRegisterDTO>> getPage(@RequestBody KlDrugRegisterPageVO klDrugRegisterPageVO) {
        IPage<KlDrugRegisterDTO> data = klDrugRegisterFacade.getPageFac(klDrugRegisterPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "删除[by:zhoutg]",
            notes = "id：主键")
    @PostMapping("/delete")
    @SysLogger("delete")
    @DSTransactional
    public RespDTO<Boolean> delete(@RequestBody KlDrugRegisterDelVO klDrugRegisterDelVO) {
        klDrugRegisterFacade.deleteByIdFac(klDrugRegisterDelVO);
        return RespDTO.onSuc(true);
    }

    @ApiOperation(value = "根据参数获取数据[by:zhoutg]",
            notes = "id: 主键")
    @PostMapping("/getByParam")
    public RespDTO<KlDrugRegisterDTO> getByParam(@RequestBody KlDrugRegisterGetVO klDrugRegisterGetVO) {
        KlDrugRegisterDTO data = klDrugRegisterFacade.getByParam(klDrugRegisterGetVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "药品注册名检索药品通用名，已经关联的需要过滤[by:zhoutg]",
            notes = "name: 检索名")
    @PostMapping("/searchDrug")
    public RespDTO<List<KlConceptSimDTO>> searchDrug(@RequestBody KlDrugSearchVO klDrugSearchVO) {
        List<KlConceptSimDTO> data = klDrugRegisterFacade.searchDrugFac(klDrugSearchVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "注册药品导入[by:zhoutg]",
            notes = "")
    @PostMapping("/importRegister")
    @DSTransactional
    public void importRegister(@RequestParam("file") MultipartFile file, HttpServletResponse response) {
        klDrugRegisterFacade.importRegister(file, response);
    }
}
